/*global require*/

function module (config, errback) {
   "use strict";

   var exports = {},
       _ = require('underscore'),
       async = require('async'),
       timer,
       log = config.log,
       exec = require('child_process').exec,
       fs = require('fs'),
       path = require('path'),
       statusData = {};

   function status (callback) {
      callback(null, statusData);
   }

   function symlink(done) {
      // config is in config.modules.symlink
      var jobs = [];
      _.each(config.modules.symlink, function (dest, src) {
         jobs.push(function (cb) {
            dest = path.join(config.paths.output, dest);
            src = path.join(config.paths.output, src);

            // try to unlink it first
            fs.unlink(src, function (err) {
               fs.symlink(dest, src, function (err) {
                  if (err) {
                     log.error('symlinking ' + src + ' to ' + dest);
                     cb(err);
                  } else {
                     cb();
                  }
               });
            });
         });
      });

      async.parallel(jobs, done);
   }

   function start () {
      log.info('symlink: setting symlinks');
      symlink();
   }

   function shutdown (done) {
      done();
   }

   exports.start = start;
   exports.shutdown = shutdown;
   exports.status = status;

   return exports;
}

exports.module = module;
